@@ -1,7 +1,7 @@
 /* parser.h -- Everything you wanted to know about the parser, but were
    afraid to ask. */
 
-/* Copyright (C) 1995-2010 Free Software Foundation, Inc.
+/* Copyright (C) 1995-2021 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -30,7 +30,7 @@
 #define PST_ALEXPNEXT	0x000002	/* expand next word for aliases */
 #define PST_ALLOWOPNBRC	0x000004	/* allow open brace for function def */
 #define PST_NEEDCLOSBRC	0x000008	/* need close brace */
-#define PST_DBLPAREN	0x000010	/* double-paren parsing */
+#define PST_DBLPAREN	0x000010	/* double-paren parsing - unused */
 #define PST_SUBSHELL	0x000020	/* ( ... ) subshell */
 #define PST_CMDSUBST	0x000040	/* $( ... ) command substitution */
 #define PST_CASESTMT	0x000080	/* parsing a case statement */
@@ -47,6 +47,10 @@
 #define PST_REPARSE	0x040000	/* re-parsing in parse_string_to_word_list */
 #define PST_REDIRLIST	0x080000	/* parsing a list of redirections preceding a simple command name */
 #define PST_COMMENT	0x100000	/* parsing a shell comment; used by aliases */
+#define PST_ENDALIAS	0x200000	/* just finished expanding and consuming an alias */
+#define PST_NOEXPAND	0x400000	/* don't expand anything in read_token_word; for command substitution */
+#define PST_NOERROR	0x800000	/* don't print error messages in yyerror */
+#define PST_STRING	0x1000000	/* parsing a string to a command or word list */
 
 /* Definition of the delimiter stack.  Needed by parse.y and bashhist.c. */
 struct dstack {
